@@ -29,6 +29,7 @@
 #include "LockScreenAcceleratorController.h"
 #include "SuspendInhibitorManager.h"
 #include "ScreenSaverDBusManager.h"
+#include "UserAuthenticator.h"
 #include "unity-shared/BackgroundEffectHelper.h"
 #include "unity-shared/UpstartWrapper.h"
 
@@ -44,7 +45,7 @@ class Controller : public sigc::trackable
 public:
   typedef std::shared_ptr<Controller> Ptr;
 
-  Controller(DBusManager::Ptr const&, session::Manager::Ptr const&,
+  Controller(DBusManager::Ptr const&, session::Manager::Ptr const&, key::Grabber::Ptr const&,
              UpstartWrapper::Ptr const& upstart_wrapper = std::make_shared<UpstartWrapper>(),
              ShieldFactoryInterface::Ptr const& shield_factory = std::make_shared<ShieldFactory>(),
              bool test_mode = false);
@@ -53,6 +54,8 @@ public:
 
   bool IsLocked() const;
   bool HasOpenMenu() const;
+  bool IsPaintInhibited() const;
+  void MarkBufferHasCleared();
 
 private:
   friend class TestLockScreenController;
@@ -76,6 +79,11 @@ private:
   void OnPresenceStatusChanged(bool idle);
   void OnScreenSaverActivationRequest(bool activate);
   void OnPrimaryShieldMotion(int x, int y);
+  void OnLockScreenInputEvent(XEvent const&);
+  void OnBlankWindowInputEvent(XEvent const&);
+
+  void InhibitPaint();
+  void UninhibitPaint();
 
   std::vector<nux::ObjectPtr<BaseShield>> shields_;
   nux::ObjectWeakPtr<BaseShield> primary_shield_;
@@ -84,11 +92,13 @@ private:
 
   DBusManager::Ptr dbus_manager_;
   session::Manager::Ptr session_manager_;
-  indicator::Indicators::Ptr indicators_;
+  menu::Manager::Ptr menu_manager_;
+  key::Grabber::Ptr key_grabber_;
   AcceleratorController::Ptr accelerator_controller_;
   UpstartWrapper::Ptr upstart_wrapper_;
   ShieldFactoryInterface::Ptr shield_factory_;
   SuspendInhibitorManager::Ptr suspend_inhibitor_manager_;
+  UserAuthenticator::Ptr user_authenticator_;
 
   nux::animation::AnimateValue<double> fade_animator_;
   nux::animation::AnimateValue<double> blank_window_animator_;
@@ -96,6 +106,8 @@ private:
   bool test_mode_;
   bool prompt_activation_;
   BlurType old_blur_type_;
+  bool is_paint_inhibited_;
+  bool buffer_cleared_;
 
   connection::Wrapper uscreen_connection_;
   connection::Wrapper hidden_window_connection_;
