@@ -18,6 +18,8 @@
 *              Andrea Azzarone <andrea.azzarone@canonical.com>
 */
 
+#include "config.h"
+
 #include <glib.h>
 #include <NuxCore/Logger.h>
 #include <UnityCore/GLibSource.h>
@@ -39,6 +41,8 @@ const std::string SETTINGS_NAME = "com.canonical.Unity";
 const std::string FORM_FACTOR = "form-factor";
 const std::string DOUBLE_CLICK_ACTIVATE = "double-click-activate";
 const std::string DESKTOP_TYPE = "desktop-type";
+const std::string PAM_CHECK_ACCOUNT_TYPE = "pam-check-account-type";
+const std::string LOWGFX = "lowgfx";
 
 const std::string LAUNCHER_SETTINGS = "com.canonical.Unity.Launcher";
 const std::string LAUNCHER_POSITION = "launcher-position";
@@ -54,6 +58,9 @@ const std::string CURSOR_SCALE_FACTOR = "cursor-scale-factor";
 const std::string APP_SCALE_MONITOR = "app-scale-factor-monitor";
 const std::string APP_USE_MAX_SCALE = "app-fallback-to-maximum-scale-factor";
 
+const std::string COMPIZ_SETTINGS = "org.compiz";
+const std::string COMPIZ_PROFILE = "current-profile";
+
 const std::string UBUNTU_UI_SETTINGS = "com.ubuntu.user-interface";
 const std::string SCALE_FACTOR = "scale-factor";
 
@@ -66,10 +73,21 @@ const std::string GNOME_TEXT_SCALE_FACTOR = "text-scaling-factor";
 const std::string REMOTE_CONTENT_SETTINGS = "com.canonical.Unity.Lenses";
 const std::string REMOTE_CONTENT_KEY = "remote-content-search";
 
+const std::string GESTURES_SETTINGS = "com.canonical.Unity.Gestures";
+const std::string LAUNCHER_DRAG = "launcher-drag";
+const std::string DASH_TAP = "dash-tap";
+const std::string WINDOWS_DRAG_PINCH = "windows-drag-pinch";
+
+const std::string CCS_PROFILE_CHANGER_TOOL = "compiz-config-profile-setter";
+const std::string CCS_PROFILE_DEFAULT = "unity";
+const std::string CCS_PROFILE_LOWGFX = CCS_PROFILE_DEFAULT + "-lowgfx";
+
 const int DEFAULT_LAUNCHER_SIZE = 64;
 const int MINIMUM_DESKTOP_HEIGHT = 800;
 const int GNOME_SETTINGS_CHANGED_WAIT_SECONDS = 1;
 const double DEFAULT_DPI = 96.0f;
+const double DPI_SCALING_LIMIT = 140.0f;
+const int DPI_SCALING_STEP = 8;
 }
 
 //
@@ -81,8 +99,10 @@ public:
   Impl(Settings* owner)
     : parent_(owner)
     , usettings_(g_settings_new(SETTINGS_NAME.c_str()))
+    , compiz_settings_(g_settings_new(COMPIZ_SETTINGS.c_str()))
     , launcher_settings_(g_settings_new(LAUNCHER_SETTINGS.c_str()))
     , lim_settings_(g_settings_new(LIM_SETTINGS.c_str()))
+    , gestures_settings_(g_settings_new(GESTURES_SETTINGS.c_str()))
     , ui_settings_(g_settings_new(UI_SETTINGS.c_str()))
     , ubuntu_ui_settings_(g_settings_new(UBUNTU_UI_SETTINGS.c_str()))
     , gnome_ui_settings_(g_settings_new(GNOME_UI_SETTINGS.c_str()))
@@ -92,10 +112,10 @@ public:
     , cached_form_factor_(FormFactor::DESKTOP)
     , cursor_scale_(1.0)
     , cached_double_click_activate_(true)
-    , changing_gnome_settings_(false)
-    , lowGfx_(false)
     , remote_content_enabled_(true)
   {
+    InitializeLowGfx();
+
     parent_->form_factor.SetGetterFunction(sigc::mem_fun(this, &Impl::GetFormFactor));
     parent_->form_factor.SetSetterFunction(sigc::mem_fun(this, &Impl::SetFormFactor));
     parent_->double_click_activate.SetGetterFunction(sigc::mem_fun(this, &Impl::GetDoubleClickActivate));
@@ -103,10 +123,21 @@ public:
     parent_->launcher_position.SetGetterFunction(sigc::mem_fun(this, &Impl::GetLauncherPosition));
     parent_->launcher_position.SetSetterFunction(sigc::mem_fun(this, &Impl::SetLauncherPosition));
     parent_->desktop_type.SetGetterFunction(sigc::mem_fun(this, &Impl::GetDesktopType));
+    parent_->pam_check_account_type.SetGetterFunction(sigc::mem_fun(this, &Impl::GetPamCheckAccountType));
+    parent_->supports_3d.changed.connect(sigc::mem_fun(this, &Impl::OnSupports3DChanged));
+    parent_->low_gfx.changed.connect(sigc::mem_fun(this, &Impl::UpdateCompizProfile));
 
     for (unsigned i = 0; i < monitors::MAX; ++i)
       em_converters_.emplace_back(std::make_shared<EMConverter>());
 
+    signals_.Add<void, GSettings*, const gchar*>(compiz_settings_, "changed::" + COMPIZ_PROFILE, [this] (GSettings*, const gchar *) {
+      parent_->low_gfx = (GetCurrentCompizProfile() == CCS_PROFILE_LOWGFX);
+    });
+
+    signals_.Add<void, GSettings*, const gchar*>(usettings_, "changed::" + LOWGFX, [this] (GSettings*, const gchar *) {
+      UpdateCompizProfile(GetLowGfxSetting());
+    });
+
     signals_.Add<void, GSettings*, const gchar*>(usettings_, "changed::" + FORM_FACTOR, [this] (GSettings*, const gchar*) {
       CacheFormFactor();
     });
@@ -149,17 +180,18 @@ public:
     });
 
     signals_.Add<void, GSettings*, const gchar*>(gnome_ui_settings_, "changed::" + GNOME_TEXT_SCALE_FACTOR, [this] (GSettings*, const gchar* t) {
-      if (!changing_gnome_settings_)
-      {
-        double new_scale_factor = g_settings_get_double(gnome_ui_settings_, GNOME_TEXT_SCALE_FACTOR.c_str());
-        g_settings_set_double(ui_settings_, TEXT_SCALE_FACTOR.c_str(), new_scale_factor);
-      }
+      double new_scale_factor = g_settings_get_double(gnome_ui_settings_, GNOME_TEXT_SCALE_FACTOR.c_str());
+      g_settings_set_double(ui_settings_, TEXT_SCALE_FACTOR.c_str(), new_scale_factor);
     });
 
     signals_.Add<void, GSettings*, const gchar*>(lim_settings_, "changed", [this] (GSettings*, const gchar*) {
       UpdateLimSetting();
     });
 
+    signals_.Add<void, GSettings*, const gchar*>(gestures_settings_, "changed", [this] (GSettings*, const gchar*) {
+      UpdateGesturesSetting();
+    });
+
     signals_.Add<void, GSettings*, const gchar*>(remote_content_settings_, "changed::" + REMOTE_CONTENT_KEY, [this] (GSettings*, const gchar* t) {
       UpdateRemoteContentSearch();
     });
@@ -168,6 +200,7 @@ public:
 
     // The order is important here, DPI is the last thing to be updated
     UpdateLimSetting();
+    UpdateGesturesSetting();
     UpdateTextScaleFactor();
     UpdateCursorScaleFactor();
     UpdateFontSize();
@@ -215,6 +248,71 @@ public:
     cached_launcher_position_ = static_cast<LauncherPosition>(g_settings_get_enum(launcher_settings_, LAUNCHER_POSITION.c_str()));
   }
 
+  void InitializeLowGfx()
+  {
+    parent_->low_gfx = GetLowGfxSetting();
+    UpdateCompizProfile(parent_->low_gfx());
+  }
+
+  std::string GetCurrentCompizProfile()
+  {
+    return glib::String(g_settings_get_string(compiz_settings_, COMPIZ_PROFILE.c_str())).Str();
+  }
+
+  glib::Variant GetUserLowGfxSetting()
+  {
+    return glib::Variant(g_settings_get_user_value(usettings_, LOWGFX.c_str()), glib::StealRef());
+  }
+
+  bool GetLowGfxSetting()
+  {
+    if (glib::Variant const& user_setting = GetUserLowGfxSetting())
+    {
+      return user_setting.GetBool();
+    }
+    else
+    {
+      auto default_profile = glib::gchar_to_string(g_getenv("UNITY_DEFAULT_PROFILE"));
+      if (!default_profile.empty())
+      {
+        return (default_profile == CCS_PROFILE_LOWGFX);
+      }
+      else if (!parent_->supports_3d() ||
+               glib::gchar_to_string(getenv("UNITY_HAS_3D_SUPPORT")) == "false")
+      {
+        return true;
+      }
+      else
+      {
+        return (GetCurrentCompizProfile() == CCS_PROFILE_LOWGFX);
+      }
+    }
+  }
+
+  void UpdateCompizProfile(bool lowgfx)
+  {
+    auto const& profile = lowgfx ? CCS_PROFILE_LOWGFX : CCS_PROFILE_DEFAULT;
+
+    if (GetCurrentCompizProfile() == profile)
+      return;
+
+    auto profile_change_cmd = (std::string(UNITY_LIBDIR G_DIR_SEPARATOR_S) + CCS_PROFILE_CHANGER_TOOL + " " + profile);
+
+    glib::Error error;
+    g_spawn_command_line_async(profile_change_cmd.c_str(), &error);
+
+    if (error)
+    {
+      LOG_ERROR(logger) << "Failed to switch compiz profile: " << error;
+    }
+  }
+
+  void OnSupports3DChanged(bool supports_3d)
+  {
+    if (!GetUserLowGfxSetting())
+      parent_->low_gfx = !supports_3d;
+  }
+
   void UpdateLimSetting()
   {
     parent_->lim_movement_thresold = g_settings_get_uint(lim_settings_, CLICK_MOVEMENT_THRESHOLD.c_str());
@@ -222,6 +320,14 @@ public:
     parent_->lim_unfocused_popup = g_settings_get_boolean(lim_settings_, UNFOCUSED_MENU_POPUP.c_str());
   }
 
+  void UpdateGesturesSetting()
+  {
+    parent_->gestures_launcher_drag = g_settings_get_boolean(gestures_settings_, LAUNCHER_DRAG.c_str());
+    parent_->gestures_dash_tap = g_settings_get_boolean(gestures_settings_, DASH_TAP.c_str());
+    parent_->gestures_windows_drag_pinch = g_settings_get_boolean(gestures_settings_, WINDOWS_DRAG_PINCH.c_str());
+    parent_->gestures_changed.emit();
+  }
+
   FormFactor GetFormFactor() const
   {
     return cached_form_factor_;
@@ -254,6 +360,11 @@ public:
     return static_cast<DesktopType>(g_settings_get_enum(usettings_, DESKTOP_TYPE.c_str()));
   }
 
+  bool GetPamCheckAccountType() const
+  {
+    return g_settings_get_boolean(usettings_, PAM_CHECK_ACCOUNT_TYPE.c_str());
+  }
+
   int GetFontSize() const
   {
     gint font_size;
@@ -297,6 +408,34 @@ public:
     return em_converters_[monitor];
   }
 
+  int FindOptimalScale(const UScreen* uscreen, const int monitor)
+  {
+    auto const& geo = uscreen->GetMonitorGeometry(monitor);
+    auto const& size = uscreen->GetMonitorPhysicalSize(monitor);
+    auto scale = DPI_SCALING_STEP;
+
+    if ((size.width == 160 && size.height == 90) ||
+        (size.width == 160 && size.height == 100) ||
+        (size.width == 16 && size.height == 9) ||
+        (size.width == 16 && size.height == 10))
+    {
+      return scale;
+    }
+
+    if (size.width > 0 && size.height > 0)
+    {
+      const double dpi_x = static_cast<double>(geo.width) / (size.width / 25.4);
+      const double dpi_y = static_cast<double>(geo.height) / (size.height / 25.4);
+
+      const auto dpi = std::max(dpi_x, dpi_y);
+
+      if (dpi > DPI_SCALING_LIMIT)
+        scale = static_cast<int>(scale * std::lround(dpi / DPI_SCALING_LIMIT));
+    }
+
+    return scale;
+  }
+
   void UpdateDPI()
   {
     auto* uscreen = UScreen::GetDefault();
@@ -307,6 +446,10 @@ public:
     glib::Variant dict;
     g_settings_get(ubuntu_ui_settings_, SCALE_FACTOR.c_str(), "@a{si}", &dict);
 
+    bool dict_changed = false;
+    GVariantBuilder builder;
+    g_variant_builder_init(&builder, G_VARIANT_TYPE("a{si}"));
+
     glib::String app_target_monitor(g_settings_get_string(ui_settings_, APP_SCALE_MONITOR.c_str()));
     double app_target_scale = 0;
 
@@ -320,8 +463,18 @@ public:
         double ui_scale = 1.0f;
         int value;
 
-        if (g_variant_lookup(dict, monitor_name.c_str(), "i", &value) && value > 0)
-          ui_scale = static_cast<double>(value)/8.0f;
+        if (g_variant_lookup(dict, monitor_name.c_str(), "i", &value))
+        {
+          if (value > 0)
+            ui_scale = static_cast<double>(value)/DPI_SCALING_STEP;
+        }
+        else
+        {
+          value = FindOptimalScale(uscreen, monitor);
+          ui_scale = static_cast<double>(value)/DPI_SCALING_STEP;
+          dict_changed = true;
+        }
+        g_variant_builder_add(&builder, "{si}", monitor_name.c_str(), value);
 
         if (app_target_monitor.Str() == monitor_name)
           app_target_scale = ui_scale;
@@ -335,6 +488,12 @@ public:
         any_changed = true;
     }
 
+    glib::Variant new_dict(g_variant_builder_end(&builder));
+    if (dict_changed)
+    {
+      g_settings_set_value(ubuntu_ui_settings_, SCALE_FACTOR.c_str(), new_dict);
+    }
+
     if (app_target_scale == 0)
       app_target_scale = (g_settings_get_boolean(ui_settings_, APP_USE_MAX_SCALE.c_str())) ? max_scale : min_scale;
 
@@ -346,9 +505,8 @@ public:
 
   void UpdateAppsScaling(double scale)
   {
-    changing_gnome_settings_ = true;
-    changing_gnome_settings_timeout_.reset();
-    unsigned integer_scaling = std::max<unsigned>(1, scale);
+    signals_.Block(gnome_ui_settings_);
+    unsigned integer_scaling = std::max<unsigned>(1, std::lround(scale));
     double point_scaling = scale / static_cast<double>(integer_scaling);
     double text_scale_factor = parent_->font_scaling() * point_scaling;
     glib::Variant default_cursor_size(g_settings_get_default_value(gnome_ui_settings_, GNOME_CURSOR_SIZE.c_str()), glib::StealRef());
@@ -358,7 +516,7 @@ public:
     g_settings_set_double(gnome_ui_settings_, GNOME_TEXT_SCALE_FACTOR.c_str(), text_scale_factor);
 
     changing_gnome_settings_timeout_.reset(new glib::TimeoutSeconds(GNOME_SETTINGS_CHANGED_WAIT_SECONDS, [this] {
-      changing_gnome_settings_ = false;
+      signals_.Unblock(gnome_ui_settings_);
       return false;
     }, glib::Source::Priority::LOW));
   }
@@ -382,8 +540,10 @@ public:
 
   Settings* parent_;
   glib::Object<GSettings> usettings_;
+  glib::Object<GSettings> compiz_settings_;
   glib::Object<GSettings> launcher_settings_;
   glib::Object<GSettings> lim_settings_;
+  glib::Object<GSettings> gestures_settings_;
   glib::Object<GSettings> ui_settings_;
   glib::Object<GSettings> ubuntu_ui_settings_;
   glib::Object<GSettings> gnome_ui_settings_;
@@ -396,8 +556,6 @@ public:
   FormFactor cached_form_factor_;
   double cursor_scale_;
   bool cached_double_click_activate_;
-  bool changing_gnome_settings_;
-  bool lowGfx_;
   bool remote_content_enabled_;
 };
 
@@ -406,7 +564,8 @@ public:
 //
 
 Settings::Settings()
-  : is_standalone(false)
+  : supports_3d(glib::gchar_to_string(getenv("UNITY_HAS_3D_SUPPORT")) != "false")
+  , is_standalone(false)
   , pimpl(new Impl(this))
 {
   if (settings_instance)
@@ -434,21 +593,6 @@ Settings& Settings::Instance()
   return *settings_instance;
 }
 
-bool Settings::GetLowGfxMode() const
-{
-  return pimpl->lowGfx_;
-}
-
-void Settings::SetLowGfxMode(const bool low_gfx)
-{
-  if (pimpl->lowGfx_ != low_gfx)
-  {
-    pimpl->lowGfx_ = low_gfx;
-
-    low_gfx_changed.emit();
-  }
-}
-
 EMConverter::Ptr const& Settings::em(int monitor) const
 {
   return pimpl->em(monitor);
