@@ -145,7 +145,12 @@ void UnityWindowView::OnClosableChanged(bool closable)
 {
   if (!closable)
   {
-    close_button_ = nullptr;
+    if (close_button_)
+    {
+      close_button_->UnParentObject();
+      close_button_ = nullptr;
+    }
+
     return;
   }
 
@@ -282,7 +287,7 @@ void UnityWindowView::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
       auto temp_background_color = background_color();
       auto blend_mode = nux::LAYER_BLEND_MODE_OVERLAY;
 
-      if (Settings::Instance().GetLowGfxMode() || BackgroundEffectHelper::blur_type == BLUR_NONE)
+      if (Settings::Instance().low_gfx() || BackgroundEffectHelper::blur_type == BLUR_NONE)
       {
         temp_background_color.alpha = 1.0f;
         blend_mode = nux::LAYER_BLEND_MODE_NORMAL;
