@@ -37,18 +37,23 @@ public:
   void Emit(std::string const& name);
 
 private:
-  glib::DBusProxy::Ptr upstart_proxy_;
+  bool test_mode_;
 };
 
 UpstartWrapper::Impl::Impl(bool test_mode)
-{
-  upstart_proxy_ = std::make_shared<unity::glib::DBusProxy>(test_mode ?  "com.canonical.Unity.Test.Upstart" : DBUS_SERVICE_UPSTART,
-                                                            DBUS_PATH_UPSTART, DBUS_INTERFACE_UPSTART);
-}
+  : test_mode_(test_mode)
+{}
 
 void UpstartWrapper::Impl::Emit(std::string const& name)
 {
-  upstart_proxy_->Call("EmitEvent", g_variant_new("(sasb)", name.c_str(), nullptr, 0));
+  auto flags = static_cast<GDBusProxyFlags>(G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+                                            G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS);
+
+  auto proxy = std::make_shared<unity::glib::DBusProxy>(test_mode_ ?  "com.canonical.Unity.Test.Upstart" : DBUS_SERVICE_UPSTART,
+                                                        DBUS_PATH_UPSTART, DBUS_INTERFACE_UPSTART,
+                                                        G_BUS_TYPE_SESSION, flags);
+
+  proxy->CallBegin("EmitEvent", g_variant_new("(sasb)", name.c_str(), nullptr, 0), [proxy] (GVariant*, glib::Error const&) {});
 }
 
 //
