@@ -31,8 +31,10 @@ using namespace testing;
 #include "lockscreen/LockScreenSettings.h"
 #include "lockscreen/ScreenSaverDBusManager.h"
 #include "unity-shared/DashStyle.h"
+#include "unity-shared/InputMonitor.h"
 #include "unity-shared/PanelStyle.h"
 #include "unity-shared/UScreen.h"
+#include "mock_key_grabber.h"
 #include "test_mock_session_manager.h"
 #include "test_uscreen_mock.h"
 #include "test_utils.h"
@@ -54,7 +56,7 @@ const unsigned TICK_DURATION =  10 * 1000;
 struct MockShield : BaseShield
 {
   MockShield()
-    : BaseShield(nullptr, nullptr, nullptr, nux::ObjectPtr<AbstractUserPromptView>(), 0, false)
+    : BaseShield(nullptr, nullptr, nux::ObjectPtr<AbstractUserPromptView>(), 0, false)
   {}
 
   MOCK_CONST_METHOD0(IsIndicatorOpen, bool());
@@ -66,7 +68,7 @@ struct MockShield : BaseShield
 struct ShieldFactoryMock : ShieldFactoryInterface
 {
   nux::ObjectPtr<BaseShield> CreateShield(session::Manager::Ptr const&,
-                                          indicator::Indicators::Ptr const&,
+                                          menu::Manager::Ptr const&,
                                           Accelerators::Ptr const&,
                                           nux::ObjectPtr<AbstractUserPromptView> const&,
                                           int, bool) override
@@ -80,19 +82,21 @@ struct TestLockScreenController : Test
   TestLockScreenController()
     : animation_controller(tick_source)
     , session_manager(std::make_shared<NiceMock<session::MockManager>>())
+    , key_grabber(std::make_shared<key::MockGrabber::Nice>())
     , dbus_manager(std::make_shared<DBusManager>(session_manager))
     , upstart_wrapper(std::make_shared<UpstartWrapper>())
     , shield_factory(std::make_shared<ShieldFactoryMock>())
-    , controller(dbus_manager, session_manager, upstart_wrapper, shield_factory)
+    , controller(dbus_manager, session_manager, key_grabber, upstart_wrapper, shield_factory)
   {}
 
   struct ControllerWrap : Controller
   {
     ControllerWrap(DBusManager::Ptr const& dbus_manager,
                    session::Manager::Ptr const& session_manager,
+                   key::Grabber::Ptr const& key_grabber,
                    UpstartWrapper::Ptr const& upstart_wrapper,
                    ShieldFactoryInterface::Ptr const& shield_factory)
-      : Controller(dbus_manager, session_manager, upstart_wrapper, shield_factory, /* test_mode */ true)
+      : Controller(dbus_manager, session_manager, key_grabber, upstart_wrapper, shield_factory, /* test_mode */ true)
     {}
 
     using Controller::shields_;
@@ -103,10 +107,12 @@ struct TestLockScreenController : Test
   nux::animation::AnimationController animation_controller;
 
   MockUScreen uscreen;
+  unity::input::Monitor im;
   unity::dash::Style dash_style;
   unity::panel::Style panel_style;
   unity::lockscreen::Settings lockscreen_settings;
   session::MockManager::Ptr session_manager;
+  key::MockGrabber::Ptr key_grabber;
   DBusManager::Ptr dbus_manager;
   unity::UpstartWrapper::Ptr upstart_wrapper;
 
@@ -123,7 +129,7 @@ TEST_F(TestLockScreenController, DisconnectUScreenSignalsOnDestruction)
 {
   size_t before = uscreen.changed.size();
   {
-    Controller dummy(dbus_manager, session_manager);
+    Controller dummy(dbus_manager, session_manager, key_grabber);
   }
   ASSERT_EQ(before, uscreen.changed.size());
 
@@ -135,7 +141,7 @@ TEST_F(TestLockScreenController, DisconnectSessionManagerSignalsOnDestruction)
 {
   size_t before = session_manager->unlock_requested.size();
   {
-    Controller dummy(dbus_manager, session_manager);
+    Controller dummy(dbus_manager, session_manager, key_grabber);
   }
   ASSERT_EQ(before, session_manager->unlock_requested.size());
 
@@ -177,7 +183,7 @@ TEST_F(TestLockScreenController, SwitchToMultiMonitor)
   session_manager->lock_requested.emit();
 
   Utils::WaitUntilMSec([this]{ return controller.shields_.size() == 1; });
-  ASSERT_EQ(1, controller.shields_.size());
+  ASSERT_EQ(1u, controller.shields_.size());
 
   Utils::WaitUntilMSec([this]{ return uscreen.GetMonitors().at(0) == controller.shields_.at(0)->GetGeometry(); });
   EXPECT_EQ(uscreen.GetMonitors().at(0), controller.shields_.at(0)->GetGeometry());
@@ -207,7 +213,7 @@ TEST_F(TestLockScreenController, SwitchToSingleMonitor)
 
   uscreen.Reset(/* emit_change */ true);
 
-  ASSERT_EQ(1, controller.shields_.size());
+  ASSERT_EQ(1u, controller.shields_.size());
   EXPECT_EQ(uscreen.GetMonitors().at(0), controller.shields_.at(0)->GetGeometry());
 }
 
@@ -216,7 +222,7 @@ TEST_F(TestLockScreenController, UnlockScreenOnSingleMonitor)
   session_manager->lock_requested.emit();
 
   Utils::WaitUntilMSec([this]{ return controller.shields_.size() == 1; });
-  ASSERT_EQ(1, controller.shields_.size());
+  ASSERT_EQ(1u, controller.shields_.size());
 
   session_manager->unlock_requested.emit();
   tick_source.tick(ANIMATION_DURATION);
@@ -244,14 +250,14 @@ TEST_F(TestLockScreenController, ShieldHasGrabAfterBlank)
   // Lock...
   session_manager->lock_requested.emit();
   Utils::WaitUntilMSec([this]{ return controller.shields_.size() == 1; });
-  ASSERT_EQ(1, controller.shields_.size());
+  ASSERT_EQ(1u, controller.shields_.size());
 
   // ...and let the screen blank.
   session_manager->presence_status_changed.emit(true);
   tick_source.tick(BLANK_ANIMATION_DURATION);
 
   ASSERT_TRUE(controller.blank_window_->GetOpacity() == 1.0);
-  ASSERT_TRUE(controller.blank_window_->OwnsPointerGrab());
+  ASSERT_FALSE(controller.blank_window_->OwnsPointerGrab());
 
   // Wake the screen
   dbus_manager->simulate_activity.emit();
